/****************************************************************
*								
*		    Contrail Member functions			
*			by Pran Mukherjee
*			pran@umich.edu
*			http://www.erim-int.com/~pran
*			04/15/00
*								
****************************************************************/
#include "Contrail.h"

Contrail::Contrail(int max_p, float pos[3], float vel[3]):
ParticleSys(max_p, pos, vel, CONTRAIL)
{
	//	fprintf(stderr, "Contrail constructor called\n");
}

Contrail::~Contrail()
{
/* do Contrail-specific deletions here, if any */
//	fprintf(stderr, "Contrail destructor called\n");
}

Contrail::S_UpdateSystem(float dt)
{
	float fade, size, mass;		
	float p_tmp[3], v_tmp[3], c_tmp[4], intensity;
	float speed = Vector3Length(S_Vel);

	if (0.0 == speed) S_Current_Max_P = 0.0f;			// When system stops, contrail is done and will fade away
	if (this->RS_Parent) S_Current_Max_P = 0.0f;	// If this is relegated to RootSystem (i.e. the anchor it's following dies), it will fade away

	// Fade rate must depend on S_Vel (so that you don't have puffs of smoke, but rather a trail.
	if ((float)S_Num_Particles < S_Current_Max_P)
	{
		for (int count = 0; count < (S_Current_Max_P * dt/5); count ++)		// Fill up in 5 seconds?  (This may need to be smoother)
		{
			fade = ((float)(rand()%90)+10.0f)/100.0f;		// Set random fade rate  (1 - 10 seconds)
			v_tmp[X] = (-2.0f * S_Vel[X]) * ((float)(rand()%20)/100.0f + 0.8f);		// 1.6 - 2.0 * system velocity, backwards
			v_tmp[Y] = (-2.0f * S_Vel[Y]) * ((float)(rand()%20)/100.0f + 0.8f);
			v_tmp[Z] = (-2.0f * S_Vel[Z]) * ((float)(rand()%20)/100.0f + 0.8f);

			 intensity = 0.5f + (float)(rand()%50)/100;
			c_tmp[R] = intensity;		// Fix this default setting
			c_tmp[G] = intensity;
			c_tmp[B] = intensity;
			c_tmp[A] = 1.0f;
			mass = 0.0005f;				// Fix this, too
			size = 3.0f + (float)(rand()%4);
			p_tmp[X] = S_Pos[X] + ((float)(rand()%60)/100.0f + 0.1f); 
			p_tmp[Y] = S_Pos[Y] + ((float)(rand()%60)/100.0f + 0.1f);
			p_tmp[Z] = S_Pos[Z] + ((float)(rand()%60)/100.0f + 0.1f);

			S_AddParticle(p_tmp, v_tmp, c_tmp, size, mass, fade, 0);		// We do NOT want these particles to have any systems attached, let alone keep them
		}
	}
	if ((0.0f == S_Current_Max_P) && (0 == S_Num_Particles)) 
	{
		if (RS_Parent) RS_Parent->RS_DeleteSystem(this);
		else if (P_Parent) P_Parent->P_DeleteSystem(this);
		else this->~Contrail();
	}
	return(true);
}
