/****************************************************************
*								
*			C++ Particle Routines				
*			by Pran Mukherjee
*			pran@umich.edu
*			http://www.erim-int.com/~pran
*			04/15/00
*
*			TO DO:
*				Maybe propagate run particle, draw particle, and update particle/anchor
*					up to ParticleSys and make Particle a friend class.   I'm not too sure about
*					this yet, though.
*								
****************************************************************/

#ifndef _Particle_h
#define _Particle_h 1

#include "ParticleSys.h"
#include "RootSystem.h"

class Particle
{
public:
	float	P_Pos[3];			// X, Y, Z location			NOTE: Both position and velocity have to be public so that
	float	P_Vel[3];			// X, Y, Z velocity					systems attached to them can update THEIR S_Pos and S_Vel

	// These next two serve as both pointers to and flags indicating ownership of the Particle (or Anchor)
	ParticleSys *S_Parent;
	RootSystem *RS_Parent;
    Particle *prev, *next;		// Doubly linked (upward and downward)

private:
	Anchor_Type A_Type;			// IF particle is an anchor (Must be attached to a RootSystem), this can be non-null
	float	P_Color[4];				// R, G, B, A				NOTE: The alpha value is also used as lifetime of the particle (modulo fade rate)
	ParticleSys *s;					// A particle can have an entire particlesys attached to it (think fireworks or flaming/smoking contrails)
	float	P_Size;						/* Particle size for glPointSize */
	float	P_Mass;					/* for real-RootSystem force calculations */
	float	P_Fade_Rate;
	int		keep_systems;		// Flag to keep its children when it dies.  (sometimes this is desireable)  Must be set per particle.  

public:
	// Constructors	
	Particle(float pos[], float vel[], float col[], float size, float mass, float fade, int keep);

	// Destructors
	~Particle();

	//  Methods
	P_AttachSystem(ParticleSys *newSys);		// Attach free-floating particle system to particle
	P_DetachSystem(ParticleSys *detSys);		// Detach system from list and repair any breach  System will be free-floating after that, so keep track of it
	P_DeleteSystem(ParticleSys *killSys);
	P_SetType(Anchor_Type a_type);

	P_RunParticle(float accel[], float force[], float dt, PS_Type type, PS_Modes out_mode);		
	P_DrawPoint();
	P_DrawQuad(GLuint image);
	P_UpdateParticle(float accel[], float force[], float dt, PS_Type type);
	P_UpdateAnchor(float accel[], float force[], float dt);
};

#endif

