/****************************************************************
*								
*			C++ Particle System Routines				
*			by Pran Mukherjee
*			pran@umich.edu
*			http://www.erim-int.com/~pran
*			04/15/00
*								
****************************************************************/

#ifndef _ParticleSys_h
#define _ParticleSys_h 1

#include "Particle.h"
#include "RootSystem.h"

class ParticleSys
{
public:
	// These next two serve as both pointers to and flags indicating ownership of the ParticleSys
	Particle *P_Parent;
	RootSystem *RS_Parent;
	ParticleSys *prev, *next;		// Can have multiple particle systems within one RootSystem system or attached to one particle (anchor) 
	Particle *p;								// Head of particle linked list  (Only public because it has to be possible to attach systems)
	PS_Modes sys_out_mode;

protected:										// Derived classes use these
	//Note that entire ParticleSys can be moving as well as individual particles within it
	//  The S_Pos and S_Vel are only used to initialize new particles within the system
	float	S_Pos[3];					// in meters
	float	S_Vel[3];					// in meters/sec
	PS_Type S_Type;

	//		Not all systems need the next things, so put them in derived classes
	int S_Num_Particles;				
	int S_Max_Particles;				
	float S_Current_Max_P;			// This is a float so that the -= fractional fade used later will function.  It should be initialized as a whole number

public:
	//	Constructor
	// Declaring position and velocity of system and system type are all necessary
	ParticleSys(int max_p, float pos[], float vel[], PS_Type type); 

	// Destructor
	virtual ~ParticleSys();							

	//  Methods		(some of these should be made private)
	S_AddParticle(float p_tmp[], float v_tmp[], float c_tmp[], float size, float mass, float fade, int keep_systems);														
	S_AttachParticle(Particle *newPart);
	S_DetachParticle(Particle *detPart);
	S_DeleteParticle(Particle *killPart);	

	S_AttachSystem(ParticleSys *newSys);		// Attach free-floating particle system to end of list that THIS is part of
																					//  Used only when a particle dies and wants to attach its systems to the parent's parent
																					//   (i.e. attach the particle's systems to the end of its parent's list)
	virtual S_UpdateSystem(float dt);					// NOTE: the dt variable isn't needed in all derived classes, but is in most.  It's a small price to pay to not 
																				  // have static global variables to deal with in this multiply-linked monolith

	S_UpdateSystemFlame();					// Write derived classes for this one, and the fountain
	S_RunSystem(float accel[], float force[], float dt);

	MakeParticleQuad();								// Build Our Particle Display List
};

AUX_RGBImageRec *LoadBMP(char *Filename);				// Loads A Bitmap Image
int LoadGLTextures(GLuint texture[]);									// Load Bitmap And Convert To Textures

#endif

