/****************************************************************
*								
*		    RootSystem Member functions			
*			by Pran Mukherjee
*			pran@umich.edu
*			http://www.erim-int.com/~pran
*			04/15/00
*								
****************************************************************/
#include "ParticleAPI.h"
#include "RootSystem.h"

friend class Particle;
friend class ParticleSys;

// CONSTRUCTOR
RootSystem::RootSystem(float accel[3], float force[3])
{
	RS_Accel[X] = accel[X];
	RS_Accel[Y] = accel[Y];
	RS_Accel[Z] = accel[Z];
	RS_Force[X] = force[X];
	RS_Force[Y] = force[Y];
	RS_Force[Z] = force[Z];

	RS_Num_Systems = 0;
	s = NULL;

	RS_Num_Anchors = 0;
	anchors = NULL;
//	fprintf(stderr, "Called fully specified RootSystem constructor\n");
}

// DESTRUCTOR
RootSystem::~RootSystem()
{
	ParticleSys *t, *c;
	c = s;
	while (NULL != c)
	{
		t = c;
		c = c->next;
		RS_DeleteSystem(t);
		free(t);
	}
	s = NULL;    
//  fprintf(stderr, "Called default RootSystem destructor\n");
}

// METHODS
RootSystem::RS_AttachParticle(Particle *newPart, Anchor_Type a_type)
{
	Particle *c = anchors;

	if (newPart->S_Parent || newPart->RS_Parent) {
		fprintf(stderr, "Particle is already attached to another RootSystem or ParticleSys.  Failing.\n");
		return(false);
	}

	if (newPart->next || newPart->prev) {
		fprintf(stderr, "Particle is still connected to previous or subsequent particles (not properly detached).  Failing.\n");
		return(false);
	}

	if (NULL != c)
	{
		while (c->next) 
		{
			c = c->next;
		}
		c->next = newPart;				//Append particle to end of list
		newPart->prev = c;				// Set new particle's backward link to former end of list
	} 
	else								// Particle is first in list
	{
		anchors = newPart;
		anchors->prev = NULL;
	}
	newPart->RS_Parent = this;
	newPart->S_Parent = NULL;
	newPart->P_SetType(a_type);
	RS_Num_Anchors++;
	return(true);
}

RootSystem::RS_DetachAnchor(Particle *detPart)
{
	if(NULL == detPart->RS_Parent) return(false);
	if (detPart->S_Parent) {
		fprintf(stderr, "This particle is attached to a ParticleSys. You made a mistake.\n");
		return(false);
	}

	if (detPart->next)			// Not end of set 
		if (detPart->prev)		// Middle of set
		{
			detPart->next->prev = detPart->prev;		
			detPart->prev->next = detPart->next;
		}		
		else								// Top of list
		{
			detPart->next->prev = NULL;
			anchors = detPart->next;
		}
	else if (detPart->prev)			// End of set
		detPart->prev->next = NULL;
	else		// Only particle attached
		anchors = NULL;

	RS_Num_Anchors --;
	detPart->RS_Parent = NULL;
	detPart->next = NULL;
	detPart->prev = NULL;
	return(true);
}

RootSystem::RS_DeleteAnchor(Particle *killPart)
{
	RS_DetachAnchor(killPart);
	killPart->~Particle();
	return(true);
 }

RootSystem::RS_AttachSystem(ParticleSys *newSys)
{
	ParticleSys *c;
	c = s;

	if (newSys->P_Parent || newSys->RS_Parent) {
		fprintf(stderr, "System is already attached to another RootSystem system or particle.  Failing.\n");
		return(false);
	}

	if (newSys->next || newSys->prev) {
		fprintf(stderr, "System is still connected to previous or subsequent systems (not properly detached).  Failing.\n");
		return(false);
	}

	if (NULL != c)
	{
		while (c->next)			// Find end of list
		{
			c = c->next;
		}
		c->next = newSys;				//Append new system to end of list
		c->next->prev = c;				// Set new system's backward link to former end of list
	} 
	else								// System is first in list
	{
		s = newSys;
		newSys->prev = NULL;
	}
	newSys->RS_Parent = this;
	newSys->P_Parent = NULL;
	RS_Num_Systems++;
	return(true);
}
	
RootSystem::RS_DetachSystem(ParticleSys *detSys)
{
	if (NULL == detSys->RS_Parent) return(false);
	if (detSys->P_Parent) {
		fprintf(stderr, "This system is attached to a particle. You made a mistake.\n");
		return(false);
	}
	if (detSys->next)			// Not end of set 
		if (detSys->prev)		// Middle of set
		{
			detSys->next->prev = detSys->prev;		
			detSys->prev->next = detSys->next;
		}		
		else								// Top of set attached to RootSystem
		{
			detSys->next->prev = NULL;
			s = detSys->next;
		}
	else if (detSys->prev)			// End of set
		detSys->prev->next = NULL;
	else		// Only system attached
		s = NULL;

	detSys->RS_Parent = NULL;
	detSys->next = NULL;
	detSys->prev = NULL;
	RS_Num_Systems--;
	return(true);
}

RootSystem::RS_DeleteSystem(ParticleSys *killSys)
{
	RS_DetachSystem(killSys);
	killSys->~ParticleSys();
	return(true);
}

RootSystem::RS_RunRootSystem(float dt)
{
	Particle *cp = anchors;
	ParticleSys *cs = s;
	static PS_Modes anchor_out;
	anchor_out.o_type = OGL_PT;		//  OGL_PT makes it a blob by default  FIX THIS!!!!!

	while (NULL != cp)																			// Always move particles first
	{
		cp->P_RunParticle(RS_Accel, RS_Force, dt, NONE, anchor_out);			// The NONE forces updateAnchor
		cp = cp->next;
	}

	while (NULL != cs)
	{
//		if (OGL_QUAD == cs->sys_out_mode.o_type) 
//			cs->MakeParticleQuad();
		cs->S_RunSystem(RS_Accel, RS_Force, dt);					// TEMPORARY;  FIX THIS
		cs = cs->next;
	}
	return(true);
}

