/****************************************************************
*								
* C++ RootSystem Routines				
* Author: Pran Mukherjee						
* Version 1.1 --- April, 2000					
*								
****************************************************************/

#ifndef _RootSystem_h
#define _RootSystem_h 1

#include "ParticleSys.h"
#include "RootSystem.h"

class RootSystem
{
private:
	Particle *anchors;				// These are meant for attached systems
	int RS_Num_Anchors;

	ParticleSys *s;
	int RS_Num_Systems;

	float	RS_Accel[3];			// in m/s^2  (This is here mainly for gravity and/or force-independant accelerations)  	This works on SYSTEMS as well as particles	
	float	RS_Force[3];			// in Newtons (kg*m/s^2)	(Wind or other things that depend on particle mass)  This works only on PARTICLES and SHARDS, not SYSTEMS

public:
	//  Constuctors
	RootSystem(float accel[], float force[]);					// Accelerations specified 

	// Destructors
	~RootSystem();

	// Methods
	RS_AttachParticle(Particle *newPart, Anchor_Type a_type);
	RS_DetachAnchor(Particle *detPart);
	RS_DeleteAnchor(Particle *killPart);	

	RS_AttachSystem(ParticleSys *newSys);			// Attach free-floating system (and any attached Particles) to RootSystem
	RS_DetachSystem(ParticleSys *detSys);			// Detach system from list and repair any breach  System will be free-floating after that, so keep track of it
	RS_DeleteSystem(ParticleSys *killSys);

	RS_RunRootSystem(float dt);									// Input size of time-step in seconds
};

#endif

