/****************************************************************
*								
*		    Smoke Member functions			
*			by Pran Mukherjee
*			pran@umich.edu
*			http://www.erim-int.com/~pran
*			04/15/00
*								
****************************************************************/
#include "Smoke.h"

Smoke::Smoke(int max_p, float max_a, float pos[3], float vel[3], float p_vel[3]):
	ParticleSys(max_p, pos, vel, SMOKE)
{
	SP_Vel_Init[X] = p_vel[X];
	SP_Vel_Init[Y] = p_vel[Y];
	SP_Vel_Init[Z] = p_vel[Z];
	S_Max_Age = max_a;
	S_Age = 0.0;
//	fprintf(stderr, "Smoke constructor called\n");
}

Smoke::~Smoke()
{
/* do Smoke-specific deletions here, if any */
//	fprintf(stderr, "Smoke destructor called\n");
}

Smoke::S_UpdateSystem(float dt)
{
	float fade, size, mass;		
	float p_tmp[3], v_tmp[3], c_tmp[4], intensity;
	S_Age += dt;
	if (S_Age > S_Max_Age) S_Current_Max_P -= (S_Max_Particles/10) * dt;		//Let the smoke wear out slowly
	if (S_Current_Max_P < 0.0f) S_Current_Max_P = 0.0f;

	if ((float)this->S_Num_Particles < S_Current_Max_P)
	{
		for (int count = 0; count < (S_Current_Max_P * dt/5); count ++)		// Fill up in 5 seconds? 
		{
			fade = (float)(rand()%40)/100.0f + 0.1f;		// Set random fade rate  (2 - 10 seconds?)

			p_tmp[X] = S_Pos[X] + ((float)(rand()%100)/100-0.5f); 
			p_tmp[Y] = S_Pos[Y] + ((float)(rand()%100)/100-0.5f);
			p_tmp[Z] = S_Pos[Z] + ((float)(rand()%100)/100-0.5f);

			v_tmp[X] = SP_Vel_Init[X] + ((float)(rand()%100)/100-0.5f) * 1.5f; 
			v_tmp[Y] = SP_Vel_Init[Y] + ((float)(rand()%100)/100-0.5f);
			v_tmp[Z] = SP_Vel_Init[Z] + ((float)(rand()%100)/100-0.5f) * 1.5f;

			 intensity = 0.25f + (float)(rand()%50)/100;
			c_tmp[R] = intensity;
			c_tmp[G] = intensity;
			c_tmp[B] = intensity;
			c_tmp[A] = 1.0f;
			mass = 0.0005f;				// Fix this, too (or do I care about this for smoke?)
			size = 3.0f + (float)(rand()%4);

			S_AddParticle(p_tmp, v_tmp, c_tmp, size, mass, fade, S_NO_KEEP);		// We do NOT want these particles to have any systems attached, let alone keep them
		}
	}
	if ((0.0f == S_Current_Max_P) && (0 == S_Num_Particles)) 
	{
		if (RS_Parent) RS_Parent->RS_DeleteSystem(this);
		else if (P_Parent) P_Parent->P_DeleteSystem(this);
		else this->~Smoke();
	}
	return(true);
}
