/****************************************************************
*								
*		    Particle System Demo Driver			
*			by Pran Mukherjee
*			pran@umich.edu
*			http://www.erim-int.com/~pran
*			04/15/00
*
*			TO DO:
*					get better timer function and frame-rate functionality
*					change random number generator to additive random table-lookup
*
****************************************************************/
#include "ParticleAPI.h"				// ParticleSys class definitions
#include "Smoke.h"
#include "Contrail.h"
#include <mmsystem.h>
#include <time.h>									// Just for random seeding					CHANGE THIS TO ADDITIVE RANDOM

void CALLBACK TimerFunc(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2);
LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Declaration For WndProc
void CheckKeys (void);

UINT		TIMER_ID, DELAY;
HDC		hDC=NULL;						// Private GDI Device Context
HGLRC	hRC=NULL;					// Permanent Rendering Context
HWND	hWnd=NULL;				// Holds Our Window Handle
BOOL	done=FALSE;								// Bool Variable To Exit Loop

bool	keys[256];					// Array Used For The Keyboard Routine
bool	active=TRUE;				// Window Active Flag Set To TRUE By Default
bool	fullscreen=TRUE;			// Fullscreen Flag Set To Fullscreen Mode By Default

static int horizontal = 0;
static float dt;
static float elapsed_time = 0.0f;			// In mSec

	/* Set defaults */
	float smoke_max_age = 10.0f;		// In seconds
	float forces[3] = {5.0f, 0.0f, 3.0f} , accel[3] = {0.0f, -9.8f, 0.0f};
	float pos[3] = {20.0f, 0.1f, -20.0f}, vel[3] = {0.0f, 0.0f, 0.0f};
	float pos2[3] = {10.0f, 0.1f, -10.0f};
	
	float smk_pvel[3] = {0.0f, 8.0f, 0.0f};
	float shot_pvel[3] = {-10.0f, 20.0f, -0.5f};

	float colors[4] = {1.0f, 0.7f, 0.0f, 1.0f};

	RootSystem *root;

GLvoid ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{
	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);									// Reset The Current Viewport
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();														// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(90.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);
    glTranslatef(0.0f, -5.0f, -8.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}


int InitGL(GLvoid)										// All Setup For OpenGL Goes Here
{
	Particle *s_anchor = new Particle(pos, shot_pvel, colors, 8.0f, 1.0f, 0.0f, S_KEEP);
	Contrail *s_contrail = new Contrail(1000, pos, vel);
//	Smoke *smoke = new Smoke(500, smoke_max_age, pos2, vel, smk_pvel);

	srand( (unsigned)time( NULL ) );

	root = new RootSystem(accel, forces);

	s_contrail->MakeParticleQuad();
//	smoke->MakeParticleQuad();

	root->RS_AttachParticle(s_anchor, SHOT);
	s_anchor->P_AttachSystem(s_contrail);
//	root->RS_AttachSystem(smoke);
	

	glClearDepth(1.0f);									// Depth Buffer Setup
	glDisable(GL_DEPTH_TEST);							// Disable Depth Testing
	glEnable(GL_BLEND);									// Enable Blending
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// Really Nice Perspective Calculations
	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);				// Really Nice Point Smoothing
//	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping

    GLfloat position[] = {0.0f, 500.0f, 1000.0f, 1.0f};
    GLfloat amb[] = {1.0f, 1.0f, 1.0f, 1.0f};
    GLfloat lmamb[] = {0.2f, 0.2f, 0.2f, 1.0f};
    GLfloat spec[] = {0.0f, 0.0f, 0.0f, 0.0f};
    GLfloat shin = {10.0f};
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

    glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
    glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
    glMaterialf(GL_FRONT, GL_SHININESS, shin);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmamb);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_DEPTH_TEST);
    glShadeModel(GL_SMOOTH);
    glColorMaterial(GL_FRONT, GL_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);

	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);			// Type Of Blending To Perform
//	glBlendFunc(GL_SRC_ALPHA,GL_ONE);					
	glEnable(GL_POINT_SMOOTH);

	return(true);										// Initialization Went OK
}

void DrawGround(GLvoid)
{
	glBegin(GL_QUADS);
	for (int x = -500; x < 500; x+= 50)
	{
		for (int z = -500; z < 500; z+= 50)
		{
			glVertex3i( x, 0, z);
			glVertex3i(-x, 0, -z);
			glVertex3i( x, 0, -z);
			glVertex3i( x, 0, z);
		}
	}
	glEnd();
}

void DrawFloor()
{
	glDisable(GL_TEXTURE_2D);
	glColor3f(1.0f, 1.0f, 1.0f);
	glBegin( GL_LINE_LOOP );
		glVertex3f( -40.0f, 0.0f,  -40.0f );
		glVertex3f( 40.0f,  0.0f,  -40.0f );
		glVertex3f( 40.0f,  0.0f,  40.0f );
		glVertex3f( -40.0f,  0.0f,  40.0f );
	glEnd();
	glBegin( GL_LINE_LOOP );
		glVertex3f( -20.0f, 0.0f,  -20.0f );
		glVertex3f( 20.0f,  0.0f,  -20.0f );
		glVertex3f( 20.0f,  0.0f,  20.0f );
		glVertex3f( -20.0f,  0.0f,  20.0f );
	glEnd();
}

void DrawBox(GLvoid)
{
	glBegin(GL_QUADS);
		glVertex3f(-100.0f,0.0f, 1000.0f);
		glVertex3f(-1000.0f,0.0f,-1000.0f);
		glVertex3f( 1000.0f,0.0f,-1000.0f);
		glVertex3f( 1000.0f,0.0f, 1000.0f);
	glEnd();
}

int DrawGLScene(GLvoid)										// Here's Where We Do All The Drawing
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer
//    glColor4f(0.5f, 0.5f, 0.5f, 1.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glRotatef((GLfloat) horizontal, 0.0f, 1.0f, 0.0f);
	DrawFloor();
	root->RS_RunRootSystem(dt);				


	if (3.0f == elapsed_time/1000)
	{
		if ( (( (float)(rand()%200) -100.0f) > 0.0f) )
		{
			Particle *f_anchor = new Particle(pos2, shot_pvel, colors, 8.0f, 1.0f, 0.0f, S_KEEP);
			Contrail *f_contrail = new Contrail(1000, pos2, vel);
			f_contrail->MakeParticleQuad();

			root->RS_AttachParticle(f_anchor, FIREWORK);
			f_anchor->P_AttachSystem(f_contrail);
		}
		else
		{
			Particle *s_anchor = new Particle(pos2, shot_pvel, colors, 8.0f, 1.0f, 0.0f, S_KEEP);
			Contrail *s_contrail = new Contrail(1000, pos2, vel);
			s_contrail->MakeParticleQuad();

			root->RS_AttachParticle(s_anchor, SHOT);
			s_anchor->P_AttachSystem(s_contrail);
		}
		elapsed_time = 0.0f;

		pos2[X] = (float)(rand()%20) -10.0f;
		Clamp (pos2[X], -10.0f, 10.0f);

		pos2[Z] = (float)(rand()%20) -30.0f;
		Clamp (pos2[Z], -30.0f, -10.0f);

		shot_pvel[X] = (float)(rand()%30) - 15.0f;
		Clamp (shot_pvel[X], -15.0f, 15.0f);

		shot_pvel[Y] += (float)(rand()%5) + 15.0f;
		Clamp (shot_pvel[Y], 5.0f, 20.0f);

		shot_pvel[Z] = (float)(rand()%4) - 2.0f;
		Clamp (shot_pvel[Z], -2.0f, 2.0f);
	}

	glFlush();
	return TRUE;											// Everything Went OK
}

GLvoid KillGLWindow(GLvoid)									// Properly Kill The Window
{
	if (fullscreen)											// Are We In Fullscreen Mode?
	{
		ChangeDisplaySettings(NULL,0);						// If So Switch Back To The Desktop
		ShowCursor(TRUE);									// Show Mouse Pointer
	}

	if (hRC)												// Do We Have A Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))						// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))							// Are We Able To Delete The RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;											// Set RC To NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))						// Are We Able To Release The DC
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;											// Set DC To NULL
	}

	if (hWnd && !DestroyWindow(hWnd))						// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;											// Set hWnd To NULL
	}
}

/*	This Code Creates Our OpenGL Window.  Parameters Are:					*
 *	title			- Title To Appear At The Top Of The Window				*
 *	width			- Width Of The GL Window Or Fullscreen Mode				*
 *	height			- Height Of The GL Window Or Fullscreen Mode			*
 *	bits			- Number Of Bits To Use For Color (8/16/24/32)			*
 *	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)	*/
 
BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	HINSTANCE	hInstance;				// Holds The Instance Of The Application
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style

	fullscreen=fullscreenflag;			// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}
	
	if (fullscreen)												// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(DEVMODE));			// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(DEVMODE);				// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;		// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;									// Return FALSE
			}
		}
	}

	if (fullscreen)												// Are We Still In Fullscreen Mode?
	{
		dwExStyle=WS_EX_APPWINDOW;								// Window Extended Style
		dwStyle=WS_POPUP;										// Windows Style
		ShowCursor(FALSE);										// Hide Mouse Pointer
	}
	else														// If Not
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;			// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW;							// Windows Style
	}

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,			// Extended Style For The Window
								"OpenGL",			// Class Name
								title,				// Window Title
								dwStyle |			// Defined Window Style
								WS_CLIPSIBLINGS |	// Required Window Style
								WS_CLIPCHILDREN,	// Required Window Style
								0, 0,				// Window Position
								width, height,		// Selected Width And Height
								NULL,				// No Parent Window
								NULL,				// No Menu
								hInstance,			// Instance
								NULL)))				// Dont Pass Anything To WM_CREATE
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		bits,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);					// Set Up Our Perspective GL Screen

	if (!InitGL())									// Initialize Our Newly Created GL Window
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	return TRUE;									// Success
}

LRESULT CALLBACK WndProc(	HWND	hWnd,			// Handle For This Window
							UINT	uMsg,			// Message For This Window
							WPARAM	wParam,			// Additional Message Information
							LPARAM	lParam)			// Additional Message Information
{
	switch (uMsg)									// Check For Windows Messages
	{
		case WM_CREATE:
			// create and start multimedia timer
			DELAY = 10;								// In mSec.  1000/delay = max frames/sec 
			dt = (float)DELAY/1000;			//  Needed to tell particle systems how far to move per frame
			TIMER_ID = timeSetEvent(DELAY, 0, (LPTIMECALLBACK)TimerFunc, 0, TIME_PERIODIC);
		break;

		case WM_TIMER:
			CheckKeys();
			SwapBuffers(hDC);					// Swap Buffers (Double Buffering)
			ValidateRect( hWnd, NULL );
			elapsed_time += DELAY;
			DrawGLScene();
		break;

		case WM_ACTIVATE:							// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;						// Program Is Active
			}
			else									// Otherwise
			{
				active=FALSE;						// Program Is No Longer Active
			}

			return 0;								// Return To The Message Loop
		}

		case WM_SYSCOMMAND:							// Intercept System Commands
		{
			switch (wParam)							// Check System Calls
			{
				case SC_SCREENSAVE:					// Screensaver Trying To Start?
				case SC_MONITORPOWER:				// Monitor Trying To Enter Powersave?
				return 0;							// Prevent From Happening
			}
			break;									// Exit
		}

		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);						// Send A Quit Message
			return 0;								// Jump Back
		}

		case WM_DESTROY:
			timeKillEvent(TIMER_ID);
			PostQuitMessage(0);
		break;

		case WM_KEYDOWN:							// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;					// If So, Mark It As TRUE
			return 0;								// Jump Back
		}

		case WM_KEYUP:								// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}

		case WM_SIZE:								// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;								// Jump Back
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

//*********************************//
// The multimedia timer function
//*********************************//
void CALLBACK TimerFunc(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	SendMessage(hWnd, WM_TIMER, 0, 0);
}
//*********************************//


int WINAPI WinMain(	HINSTANCE	hInstance,			// Instance
					HINSTANCE	hPrevInstance,		// Previous Instance
					LPSTR		lpCmdLine,			// Command Line Parameters
					int			nCmdShow)			// Window Show State
{
	MSG		msg;									// Windows Message Structure

	// Ask The User Which Screen Mode They Prefer
	if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO)
	{
		fullscreen=FALSE;							// Windowed Mode
	}

	// Create Our OpenGL Window
	if (!CreateGLWindow("Blow up North Campus",800,600,16,fullscreen))
	{
		return 0;									// Quit If Window Was Not Created
	}

	if (fullscreen)									// Are We In Fullscreen Mode
	{
//		slowdown=1.0f;								// If So, Speed Up The Particles (3dfx Issue)
	}

	DrawGLScene();

	while(!done)									// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))	// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)				// Have We Received A Quit Message?
			{
				done=TRUE;							// If So done=TRUE
			}
			else									// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);				// Translate The Message
				DispatchMessage(&msg);				// Dispatch The Message
			}
		}
	}

	// Shutdown
	KillGLWindow();									// Kill The Window
	return (msg.wParam);							// Exit The Program
}

void CheckKeys (void) {

	if (!active || keys[VK_ESCAPE])	// Active?  Was There A Quit Received?
	{
		done=TRUE;							// ESC or DrawGLScene Signalled A Quit
	}
	else									// Not Time To Quit, Update Screen
	{
/*
		if (keys[VK_ADD] && (slowdown>1.0f)) slowdown-=0.01f;		// Speed Up Particles
		if (keys[VK_SUBTRACT] && (slowdown<4.0f)) slowdown+=0.01f;	// Slow Down Particles

		if (keys[VK_PRIOR])	zoom+=0.1f;		// Zoom In
		if (keys[VK_NEXT])	zoom-=0.1f;		// Zoom Out
*/
		if (keys[VK_RETURN])			// Return Key Pressed
		{
		}
/*
		if (keys[VK_UP])
		{
			vertical += 5;
			if (vertical > 90) vertical = 90;
		}

		if (keys[VK_DOWN])
		{	
			vertical -= 5;
			if (vertical < 0) vertical = 0;
		}
*/
		if (keys[VK_RIGHT]) horizontal = (horizontal +5) % 360;
		if (keys[VK_LEFT]) 	  horizontal = (horizontal - 5) % 360;
/*
		if ( (keys[' '] && !sp) || (rotate_flag && (rotate % 60)) )		// Space key pressed (FIRE!) 
		{
			sp = true;
			rotate_flag = true;
			rotate = (rotate + 5) % 360;
			if (!(rotate%60)) rotate_flag = false;				// Six barrels * 60 degrees = 360 degrees
		}
		if (!keys[' ']) sp = false;		// Release flag
*/
	}
}